/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: expr_iter.h 23 2003-10-28 02:24:16Z drory $
 * ---------------------------------------------------------------------
 *
 * Copyright (C) 2000-2002 Niv Drory <drory@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 * ---------------------------------------------------------------------
 *
 */


// This is a special version of an iterator useful for iterating over
// a TExpr<A,N> object, which is an iterator by itself, but which would
// be very inefficient to use directly, since it delegates ALL calls to
// ALL subnodes in an expression parse-tree. Here we define an ExprIter
// object which uses the 'remote-control' mechanism of the expr to iterate
// efficiently. See also comments in marray_iter.h and eval.h.

#ifndef __LTL_MARRAY__
#error "<ltl/marray/expr_iter.h> must be included via <ltl/marray.h>, never alone!"
#endif



#ifndef __LTL_EXPR_ITER__
#define __LTL_EXPR_ITER__

template<class Expr, int N>
class TExpr;


template<class A, int N>
class ExprIter
{
   public:
      typedef std::forward_iterator_tag       iterator_category;
      typedef typename TExpr<A,N>::value_type value_type;
      typedef          int                    difference_type;
      typedef const value_type&               const_reference;
      typedef const value_type*               const_pointer;
      typedef       value_type&               reference;
      typedef       value_type*               pointer;

      ExprIter( TExpr<A,N>& e )
            : e_(e), i_(*e.shape())
      { }

      ExprIter( TExpr<A,N>& e, bool end )
            : e_(e), i_()
      { }

      ExprIter( const ExprIter<A,N>& other )
            : e_(other.e_), i_(other.i_)
      { }

      inline ExprIter<A,N>& operator++()
      {
         i_.advance();
         e_.advance();
         if( i_.needAdvanceDim() )
         {
            i_.advanceDim();
            e_.advanceDim();
         }
         return *this;
      }

      void operator++( int )
      {
         ++(*this);
      }

      inline bool done() const
      {
         return i_.done();
      }

      inline value_type operator*() const
      {
         return *e_;
      }

      // FUDGE! but works for comparison with end iterator in STL algorithms
      inline bool operator==( const ExprIter<A,N>& other ) const
      {
         return done();
      }

   protected:
      TExpr<A,N>& e_;
      ShapeIter<N> i_;
};


#endif

