/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: marray_ops.h 23 2003-10-28 02:24:16Z drory $
 * ---------------------------------------------------------------------
 *
 * Copyright (C) 2000-2002 Niv Drory <drory@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 * ---------------------------------------------------------------------
 *
 */

#ifndef __LTL_IN_FILE_MARRAY__
#error "<ltl/marray/marray_ops.h> must be included via <ltl/marray.h>, never alone!"
#endif


#ifndef __LTL_MARRAYOPS__
#define __LTL_MARRAYOPS__


// --------------------------------------------------------------------
//
// To be able to use a single implementation for the evaluation of
// expressions for assignment AND operator X=, we need a little trick.
// We make the evaluation routine a template of the assignment operation
// which we define below in separate objects to handle each operatorX=
//
// --------------------------------------------------------------------

// this is just to keep everything together in class browsers
class _assignment_base
   { }
;

#define MAKE_ASSIGNMENT_OP(name,op)                             \
template<class X, class Y>                                      \
class name : public _assignment_base                            \
{                                                               \
   public:                                                      \
      static inline void assign_op( X& restrict_ x, Y y )       \
      { x op (X)y; }                                               \
}

template<class X, class Y>
class ltl_eq_assign : public _assignment_base
{
   public:
      static inline void assign_op( X& restrict_ x, Y y )
      {
         x = (X)y;
      }
};


MAKE_ASSIGNMENT_OP(ltl_plu_assign, += );
MAKE_ASSIGNMENT_OP(ltl_min_assign, -= );
MAKE_ASSIGNMENT_OP(ltl_mul_assign, *= );
MAKE_ASSIGNMENT_OP(ltl_div_assign, /= );
MAKE_ASSIGNMENT_OP(ltl_mod_assign, %= );
MAKE_ASSIGNMENT_OP(ltl_xor_assign, ^= );
MAKE_ASSIGNMENT_OP(ltl_and_assign, &= );
MAKE_ASSIGNMENT_OP(ltl_bor_assign, |= );
MAKE_ASSIGNMENT_OP(ltl_sle_assign, <<= );
MAKE_ASSIGNMENT_OP(ltl_sri_assign, >>= );



// --------------------------------------------------------------------
// OPERATORS
// --------------------------------------------------------------------

#define MAKE_EXPR_ASSIGNEMNT_OP(op,name)                                \
template<class T, int N>                                                \
template<class Expr>                                                    \
inline MArray<T,N>&                                                     \
MArray<T,N>::operator op( TExpr<Expr,N> e )                             \
{                                                                       \
    LTL_ASSERT( memBlock_, "Assignment to uninitialized MArray!" );     \
                                                                        \
    eval_assign_expr( *this, e,                                         \
                      name<T,typename TExpr<Expr,N>::value_type>() );   \
    return *this;                                                       \
}

MAKE_EXPR_ASSIGNEMNT_OP(+=, ltl_plu_assign )
MAKE_EXPR_ASSIGNEMNT_OP(-=, ltl_min_assign )
MAKE_EXPR_ASSIGNEMNT_OP(*=, ltl_mul_assign )
MAKE_EXPR_ASSIGNEMNT_OP(/=, ltl_div_assign )
MAKE_EXPR_ASSIGNEMNT_OP(%=, ltl_mod_assign )
MAKE_EXPR_ASSIGNEMNT_OP(^=, ltl_xor_assign )
MAKE_EXPR_ASSIGNEMNT_OP(&=, ltl_and_assign )
MAKE_EXPR_ASSIGNEMNT_OP(|=, ltl_bor_assign )
MAKE_EXPR_ASSIGNEMNT_OP(<<=,ltl_sle_assign )
MAKE_EXPR_ASSIGNEMNT_OP(>>=,ltl_sri_assign )



#define MAKE_ARRAY_ASSIGNEMNT_OP(op,name)                               \
template<class T, int N>                                                \
template<class T2>                                                      \
inline MArray<T,N>&                                                     \
MArray<T,N>::operator op( const MArray<T2,N>& a )                       \
{                                                                       \
    LTL_ASSERT( memBlock_, "Assignment to uninitialized MArray!" );     \
    TExpr<typename MArray<T2,N>::ConstIterator,N> i( a.begin() );       \
                                                                        \
    eval_assign_expr( *this, i, name<T,T2>() );                         \
    return *this;                                                       \
}

MAKE_ARRAY_ASSIGNEMNT_OP(+=, ltl_plu_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(-=, ltl_min_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(*=, ltl_mul_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(/=, ltl_div_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(%=, ltl_mod_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(^=, ltl_xor_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(&=, ltl_and_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(|=, ltl_bor_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(<<=,ltl_sle_assign )
MAKE_ARRAY_ASSIGNEMNT_OP(>>=,ltl_sri_assign )



#define MAKE_LITERAL_ASSIGNEMNT_OP(op,name)                             \
template<class T, int N>                                                \
inline MArray<T,N>&                                                     \
MArray<T,N>::operator op( const T t )                                   \
{                                                                       \
   LTL_ASSERT( memBlock_, "Assignment to uninitialized MArray!" );      \
   TExpr<TExprLiteral<T>,N> e(t);                                       \
                                                                        \
   eval_assign_expr( *this, e, name<T,T>() );                           \
   return *this;                                                        \
}

MAKE_LITERAL_ASSIGNEMNT_OP(+=, ltl_plu_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(-=, ltl_min_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(*=, ltl_mul_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(/=, ltl_div_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(%=, ltl_mod_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(^=, ltl_xor_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(&=, ltl_and_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(|=, ltl_bor_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(<<=,ltl_sle_assign )
MAKE_LITERAL_ASSIGNEMNT_OP(>>=,ltl_sri_assign )



#endif
