/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: memblock.h 23 2003-10-28 02:24:16Z drory $
 * ---------------------------------------------------------------------
 *
 * Copyright (C) 2000-2002 Niv Drory <drory@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 * ---------------------------------------------------------------------
 *
 */


#ifndef __LTL_MEMBLOCK__
#define __LTL_MEMBLOCK__

#include <ltl/config.h>

LTL_BEGIN_NAMESPACE

// Class MemoryBlock provides a reference-counted block of memory.  This block
// may be referred to by multiple array objects.  The memory
// is automatically deallocated when the last referring object is destructed.
template<class T>
class MemoryBlock
{


   public:
      typedef T value_type;

      MemoryBlock()
         : data_(0), references_(0)
      { }

      MemoryBlock( const int items )
         : data_(new value_type[items]), references_(0)
      { }

      MemoryBlock( value_type* restrict_ data )
         : data_(data), references_(0)
      { }

      virtual ~MemoryBlock()
      {
         if( data_ )
            delete [] data_;
      }

      void addReference()
      {
         ++references_;
      }

      value_type* restrict_ data()
      {
         addReference();
         return data_;
      }

      value_type* restrict_ data() const
      {
         addReference();
         return data_;
      }

      void removeReference()
      {
         --references_;
         if( !references_ )
         {
            delete this;
         }
      }

      int references() const
      {
         return references_;
      }

      void describeSelf() const
      {
         cout << "  MemBlock at         : " << this << endl;
         cout << "     References       : " << references_ << endl;
         cout << "     Data ptr         : " << data_ << endl;
      }

   private:   // Data members
      value_type*   restrict_ data_;
      int           references_;
};


LTL_END_NAMESPACE

#endif // __LTL_MEMBLOCK__
