/* -*- C++ -*-
 *
 * globals.h
 * Declaration of global parameters class
 *
 * --------------------------------------------------------------------
 * $Id: globals.h 37 2004-10-06 21:12:28Z drory $
 * --------------------------------------------------------------------
 *
 * Copyright (C) 2000-2001 Niv Drory <drory@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 */

#ifndef __YODA_GLOBALS__
#define __YODA_GLOBALS__

#include <string>
#include <list>
#include <ltl/marray.h>

using namespace std;

class Object;
class PhotData;

using ltl::MArray;
typedef MArray<float,2> MImage;


class Globals
{
   public:
      char** argv_;             // for logging into output files ...
      int    argc_;      
      bool   help_;             // display help string?
      bool   version_;          // display version string?
      
      string infile_;           // input image

      // detection
      bool   detect_;           // run detection pass?
      string objcat_;           // object catalog file
      float  seeing_, thresh_;  // seeing FWHM and threshold
      int    npixmin_;          // min number of pix for object
      int    splitnpix_;        // min number of pix for split object
      float  nanvalue_;         // value used to indicate INDEF pixels
      float  satvalue_;         // saturation value
      int    maxsize_;          // maxmum linear size of objects
      float  magzero_;          // magnitude zero-point
      int    nskybins_;         // number of bins in background histograms
      int    bggrid_;           // width of background grid cell
      bool   bg_iter_;          // iterate background estimate
      bool   bg_dump_images_;   // dump background and rms images
      bool   bg_dump_mask_;     // dump detection mask
      float  fracsplit_;        // luminosity stepsize for splitting
      float  splitrange_[2];    // min,max luminosity for splitting

      // psf estimation
      bool   psf_;              // generate psf template?
      string psffile_;          // filename for PSF image
      string psfinputcat_;      // catalog of objects for PSF
      string psfstarcat_;       // output catalog of objects for PSF
      int    psfminstars_;      // minimum number of stars for PSF
      int    psftemplsize_;     // size of PSF image in pixels
      float  psfcorelum_;       // min core luminosity above detection isophote
      float  psfmaxasym_;       // max asymmetry
      float  psfrmsir1_;        // max rms(ir1)/<ir1> for automatic PSF

      // photometry
      bool   photometry_;       // perform photometry?
      string photcat_;          // output filename
      string mastercat_;        // master catalog (none -> use detected objecs)
      string coordscat_;        // coordinates file (none -> use mastercat)
      float  coordscolumns_[2]; // columns of x and y coordinates
      float  aptransf_[4];      // linear transform for apertures
      bool   iraftrans_;        // tranf in iraf format: xmag,ymag,xrot,yrot?
      float  fixed_aper_;       // fixed aperture diameter in pixels
      bool   no_kron_ap_;       // don't perform kron-aperture photometry
      float  kronfactor_;       // kron factor
      float  minaperture_;      // minimum aperture size
      float  skyann_[2];        // distance and width of sky annulus
      float  gain_;             // gain in e-/ADU
      string gainkey_;          // gain keyword

      // classification
      bool   class_;            // perform classification
      string psftemplate_;      // file containing PSF template
      string classcat_;         // output catalog for classification

      // images
      MImage bg_, rms_;         // background and rms maps
      MImage psfimage_;         // PSF image
      MImage Image_;            // the background subtracted image
      list<Object*> objlist_;   // list of detected,split,and evaluated objects
      list<PhotData*> photlist_;// list with photometry results
};

#endif
