/* -*- C++ -*-
 *
 * yoda.h
 * Definitions and prototypes for yoda
 *
 * --------------------------------------------------------------------
 * $Id: yoda.h 37 2004-10-06 21:12:28Z drory $
 * --------------------------------------------------------------------
 *
 * Copyright (C) 2000-2001 Niv Drory <drory@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA 
 *
 */


#ifndef __YODA__
#define __YODA__

#include <string>
#include <list>

using namespace std;

class Object;
class PhotData;

namespace ltl {
   class FitsIn;
   class FitsOut;
   template<class T, int N> class MArray;
}
using ltl::MArray;
using ltl::FitsIn;
using ltl::FitsOut;

typedef MArray<float,2> MImage;
typedef MArray<char,2> MMask;
typedef MArray<float,1> MVector;

// detect.cpp
void detect();
void determine_sky();

// convolve.cpp
MImage convolve( const MImage& a, const float sigma, const float nan );

// background.cpp
void background( MImage& I, MImage& rms, MImage& bg );

// bg.cpp
void backgroundMap( const MImage& image, MImage& bgrms, MImage& bg, 
		    bool quiet=false );

// assemble.cpp
list<Object*> assemble( MMask& M, const MImage& I, 
			const MImage& rms, 
			const int border,
			const bool quiet=false );

// split.cpp
list<Object*> split( list<Object*> l );
void delete_list( list<Object*> l );

// evaluate.cpp
list<Object*> evaluate( const list<Object*> l, const MImage& I );

// makepsf.cpp
void makepsf( MImage& I, list<Object*> l );

// findstars.cpp
list<Object*> findstars( list<Object*> l, float sigma );
list<Object*> readstars( const char *filename, list<Object*> l );

// psf.cpp
MImage psf( list<Object*> stars, const MImage& image );

// io.cpp
string prepend_filename( string path, const string& prefix );
MImage read_fits_image( FitsIn& infits, const bool quiet=false );
void write_master_catalog( list<Object*> l, string& filename );
void write_photometry_catalog( list<PhotData*> l, string& filename );
void write_psffile( const MImage& psf );
void write_classify_catalog( list<PhotData*> l, string& filename );

//interp.cpp
float bilin_interp( const MImage& I, const int x, const int y, 
		    const int mesh );
float interp2d( const MImage& I, const float x, const float y );

// photometry.cpp
void photometry();

// apphot.cpp
void kron_aperture_phot( const MImage& Image, list<PhotData*>& pl,
			 const float k, const float minap, 
                         const float sky1, const float sky2, 
			 const float gain, const float magzero,
			 const float nanval );
void fixed_aperture_phot( const MImage& Image, list<PhotData*>& pl,
                          const float aper, 
                          const float sky1, const float sky2,
                          const float gain, const float magzero, 
                          const float nanval );

//classify.cpp
void classify();


// resolution.cpp
void resolution_classifier( MImage& Image, MImage& Psf, 
                            list<PhotData*>& pl, const float gain );

#endif

